==> prayer-1.0.12.tar.gz: prayer-1.0.12/prayer/cmd_redirect.c <==
/* $Cambridge: hermes/src/prayer/prayer/cmd_redirect.c,v 1.5 2003/10/23 15:53:05 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void generate_form(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct account *account = session->account;

    /* Titlebar */
    html_start(session, b, "Mail Redirection");
    html_banner_toolbar(session, b, "redirect", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Mail Redirection</h2>" CRLF);

    /* Form body */
    html_form_start(session, b, "POST", "redirect");

    /* Start table */
    bputs(b, "<table>" CRLF);

    /* use_redirect */
    bputs(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Enable Redirection", "use_redirect",
                             account->redirect_enabled);
    bputs(b, "</tr>" CRLF);

    /* redirect_addr */
    bputs(b, "<tr>" CRLF);
    bputs(b, "<td>&nbsp;</td>" CRLF);
    if (account->redirect_address)
        html_form_table_field(b, "Redirect address:", "redirect_addr",
                              account->redirect_address, 40);
    else
        html_form_table_field(b, "Redirect address:", "redirect_addr", "",
                              40);

    bputs(b, "</tr>" CRLF);

    /* save_copy */
    bputs(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Save copy in this account", "save_copy",
                             account->redirect_copy);
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);

    /* Submit and cancel buttons */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Options Screen");
    html_form_table_submit(b, "sub_apply", "Apply");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);
    html_form_end(b);

    html_end(b);
    response_html(request, 200);        /* Success */
}

void cmd_redirect(struct session *session)
{
    struct request *request = session->request;
    struct assoc *h = NIL;
    struct account *account = session->account;
    char *s;
    BOOL rc = T;
    ADDRESS *addr = NIL;

    if (request->method != POST) {
        if (!account_mail_check(account, request->pool)) {
            char *msg = account_fetch_message(account);
            struct buffer *b = request->write_buffer;

            /* Title bar */
            html_start(session, b, "Mail Redirection");
            html_banner_toolbar(session, b, "redirect", 0);

            /* Title line */
            bputs(b, "<h2 align=\"center\">Mail Redirection</h2>" CRLF);

            bputs(b, "<b>Error:</b> ");
            if (msg)
                html_quote_string(b, msg);
            else
                bputs(b,
                      "Unable to check mail processing status with server");
            bputs(b, "" CRLF);

            /* Form body */
            html_form_start(session, b, "POST", "redirect");

            /* Cancel buttons */
            bputs(b, "<table>" CRLF);
            bputs(b, "<tr>" CRLF);
            html_form_table_submit(b, "sub_cancel",
                                   "Back to Options screen");
            bputs(b, "</tr>" CRLF);
            bputs(b, "</table>" CRLF);
            html_form_end(b);

            html_end(b);
            response_html(request, 200);
            return;
        }
        generate_form(session);
        return;
    }

    request_decode_post(request);
    h = request->form;

    if ((s = assoc_lookup(h, "sub_apply"))) {
        account->redirect_enabled =
            (assoc_lookup(h, "use_redirect")) ? T : NIL;
        account->redirect_copy = (assoc_lookup(h, "save_copy")) ? T : NIL;

        if ((s = assoc_lookup(h, "redirect_addr")))
            string_strdup(&account->redirect_address, s);
        else
            string_strdup(&account->redirect_address, "");

        if (account->redirect_enabled) {
            addr = addr_parse(request->pool, account->redirect_address, "");

            if (!addr) {
                session_message(session, "Redirection Address invalid: %s",
                                ml_errmsg());
                rc = NIL;
            } else {
                if ((addr->next) || (addr->personal && addr->personal[0]) ||
                    (!(addr->host && addr->host[0]))) {
                    session_message(session,
                                    ("Redirection Address must be "
                                     "single, simple "
                                     "and fully qualified email address"));
                    rc = NIL;
                }
                mail_free_address(&addr);
            }

            if (rc == NIL) {
                generate_form(session);
                return;
            }
        }

        /* Update */
        if (account_mail_update(session->account, request->pool))
            session_message(session, "Updated redirection");
        else
            session_message(session, "Unable to update redirection: %s",
                            account_fetch_message(session->account));
    }

    session_redirect(session, request, "manage");
}

==> prayer-1.0.16.tar.gz: prayer-1.0.16/prayer/cmd_redirect.c <==
/* $Cambridge: hermes/src/prayer/prayer/cmd_redirect.c,v 1.9 2005/08/10 09:48:34 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void generate_form(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct account *account = session->account;

    /* Titlebar */
    html_start(session, b, "Mail Redirection");
    html_banner_toolbar(session, b, "redirect", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Mail Redirection</h2>" CRLF);

    bputs(b, "<strong>WARNING:</strong>"CRLF);
    bputs(b, "<blockquote>"CRLF);
    bputs(b, " Support staff within the University and Colleges"CRLF);
    bputs(b, " will not be able to help you with email problems if you"CRLF);
    bputs(b, " forward your email outside Cambridge and the problems"CRLF);
    bputs(b, " involve external systems. Note that there may be issues"CRLF);
    bputs(b, " of privacy and/or reliability with external email systems"CRLF);
    bputs(b, "which you should consider."CRLF);
    bputs(b, "</blockquote>"CRLF);

    /* Form body */
    html_form_start(session, b, "POST", "redirect");

    /* Start table */
    bputs(b, "<table>" CRLF);

    /* use_redirect */
    bputs(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Enable Redirection", "use_redirect",
                             account->redirect_enabled);
    bputs(b, "</tr>" CRLF);

    /* redirect_addr */
    bputs(b, "<tr>" CRLF);
    bputs(b, "<td>&nbsp;</td>" CRLF);
    if (account->redirect_address)
        html_form_table_field(b, "Redirect address:", "redirect_addr",
                              account->redirect_address, 40);
    else
        html_form_table_field(b, "Redirect address:", "redirect_addr", "",
                              40);

    bputs(b, "</tr>" CRLF);

    /* save_copy */
    bputs(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Save copy in this account", "save_copy",
                             account->redirect_copy);
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);

    /* Submit and cancel buttons */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Options Screen");
    html_form_table_submit(b, "sub_apply", "Apply");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);
    html_form_end(b);

    html_end(b);
    response_html(request, 200);        /* Success */
}

void cmd_redirect(struct session *session)
{
    struct request *request = session->request;
    struct assoc *h = NIL;
    struct account *account = session->account;
    char *s;
    BOOL rc = T;
    ADDRESS *addr = NIL;

    if (request->method != POST) {
        if (!account_mail_check(account, request->pool)) {
            char *msg = account_fetch_message(account);
            struct buffer *b = request->write_buffer;

            /* Title bar */
            html_start(session, b, "Mail Redirection");
            html_banner_toolbar(session, b, "redirect", 0);

            /* Title line */
            bputs(b, "<h2 align=\"center\">Mail Redirection</h2>" CRLF);

            bputs(b, "<b>Error:</b> ");
            if (msg) {
                html_quote_string(b, msg);
                session_log(session, "[cmd_redirect] %s", msg);
            } else {
                bputs(b,
                      "Unable to check mail processing status with server");
            }
            bputs(b, "" CRLF);

            /* Form body */
            html_form_start(session, b, "POST", "redirect");

            /* Cancel buttons */
            bputs(b, "<table>" CRLF);
            bputs(b, "<tr>" CRLF);
            html_form_table_submit(b, "sub_cancel",
                                   "Back to Options screen");
            bputs(b, "</tr>" CRLF);
            bputs(b, "</table>" CRLF);
            html_form_end(b);

            html_end(b);
            response_html(request, 200);
            return;
        }
        generate_form(session);
        return;
    }

    request_decode_post(request);
    h = request->form;

    if ((s = assoc_lookup(h, "sub_apply"))) {
        account->redirect_enabled =
            (assoc_lookup(h, "use_redirect")) ? T : NIL;
        account->redirect_copy = (assoc_lookup(h, "save_copy")) ? T : NIL;

        if ((s = assoc_lookup(h, "redirect_addr")))
            string_strdup(&account->redirect_address, s);
        else
            string_strdup(&account->redirect_address, "");

        if (account->redirect_enabled) {
            addr = addr_parse(request->pool, account->redirect_address, "");

            if (!addr) {
                session_message(session, "Redirection Address invalid: %s",
                                ml_errmsg());
                rc = NIL;
            } else {
                if ((addr->next) || (addr->personal && addr->personal[0]) ||
                    (!(addr->host && addr->host[0]))) {
                    session_message(session,
                                    ("Redirection Address must be "
                                     "single, simple "
                                     "and fully qualified email address"));
                    rc = NIL;
                }
                mail_free_address(&addr);
            }

            if (rc == NIL) {
                generate_form(session);
                return;
            }
        }

        /* Update */
        if (account_mail_update(account, request->pool))
            session_message(session, "Updated redirection");
        else {
            char *msg = account_fetch_message(account);

            session_message(session, "Unable to update redirection: %s", msg);
            session_log(session,
                        "[cmd_redirect] Unable to update redirection: %s", msg);
        }
    }

    session_redirect(session, request, "manage");
}
